"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenerateOpenapiProvider = void 0;
const ts_morph_1 = require("ts-morph");
const path_1 = require("path");
const core_1 = require("@angular-devkit/core");
const const_1 = require("./const");
const { dasherize, classify } = core_1.strings;
function GenerateOpenapiProvider(project, operatorIdList, options) {
    const sourceFile = project.createSourceFile('open-api-providers.ts');
    const providerList = [];
    sourceFile.addImportDeclaration({
        moduleSpecifier: '@angular/core',
        namedImports: ['Provider']
    });
    for (const operationId of operatorIdList) {
        if (!options.skipRemoteMethod) {
            const name = [operationId, const_1.REMOTE_METHOD_FILE_SUFFIX].join('.');
            const moduleSpecifier = './' + (0, path_1.join)(const_1.REMOTE_METHOD_BASE_PATH, dasherize(name));
            const className = classify(name.replace(/\./g, '-'));
            sourceFile.addImportDeclaration({
                moduleSpecifier,
                namedImports: [className]
            });
            providerList.push(className);
        }
    }
    sourceFile.addVariableStatement({
        declarationKind: ts_morph_1.VariableDeclarationKind.Const,
        isExported: true,
        declarations: [
            {
                name: 'OPEN_API_PROVIDERS',
                type: 'Provider[]',
                initializer: `[${providerList.join(', ')}]`,
            }
        ]
    });
}
exports.GenerateOpenapiProvider = GenerateOpenapiProvider;
//# sourceMappingURL=generate-openapi-provider.js.map