"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenerateRemoteMethod = void 0;
const tslib_1 = require("tslib");
const schematics_open_api_1 = require("@rxap/schematics-open-api");
const path_1 = require("path");
const ts_morph_1 = require("ts-morph");
const core_1 = require("@angular-devkit/core");
const create_directive_1 = require("./create-directive");
const const_1 = require("./const");
const { dasherize, classify } = core_1.strings;
function GenerateRemoteMethod(parameter) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const operationId = parameter.operationId;
        const name = [operationId, const_1.REMOTE_METHOD_FILE_SUFFIX].join('.');
        const fileName = (0, path_1.join)(const_1.REMOTE_METHOD_BASE_PATH, dasherize(name) + '.ts');
        const sourceFile = parameter.project.createSourceFile(fileName);
        const importStructures = [
            {
                moduleSpecifier: '@rxap/open-api/remote-method',
                namedImports: [
                    { name: 'RxapOpenApiRemoteMethod' },
                    { name: 'OpenApiRemoteMethodParameter' }
                ]
            },
            {
                moduleSpecifier: '@angular/core',
                namedImports: [{ name: 'Injectable' }]
            }
        ];
        switch (parameter.options.transport) {
            case 'amplify':
                importStructures.push({
                    moduleSpecifier: '@rxap/amplify-open-api',
                    namedImports: [{ name: 'AmplifyOpenApiRemoteMethod' }]
                });
                break;
            default:
                importStructures.push({
                    moduleSpecifier: '@rxap/open-api/remote-method',
                    namedImports: [{ name: 'OpenApiRemoteMethod' }]
                });
                break;
        }
        const responseType = (0, schematics_open_api_1.GetResponseType)(parameter);
        const parameterType = (0, schematics_open_api_1.GetParameterType)(parameter);
        const requestBodyType = (0, schematics_open_api_1.GetRequestBodyType)(parameter);
        if (!['void', 'any'].includes(responseType)) {
            importStructures.push({
                moduleSpecifier: `../responses/${dasherize(responseType.replace(/Response$/, ''))}.response`,
                namedImports: [{ name: responseType }]
            });
        }
        if (!['void', 'any'].includes(parameterType)) {
            importStructures.push({
                moduleSpecifier: `../parameters/${dasherize(parameterType.replace(/Parameter$/, ''))}.parameter`,
                namedImports: [{ name: parameterType }]
            });
        }
        if (!['void', 'any'].includes(requestBodyType)) {
            importStructures.push({
                moduleSpecifier: `../request-bodies/${dasherize(requestBodyType.replace(/RequestBody$/, ''))}.request-body`,
                namedImports: [{ name: requestBodyType }]
            });
        }
        const callMethodParameters = [];
        const withoutParametersAndRequestBody = parameterType === 'void' && requestBodyType === 'void';
        if (!withoutParametersAndRequestBody) {
            callMethodParameters.push({
                name: 'parameters',
                type: `OpenApiRemoteMethodParameter<${parameterType}, ${requestBodyType}>`
            });
        }
        const classStructure = {
            name: classify(name.replace(/\./g, '-')),
            decorators: [
                {
                    name: 'Injectable',
                    arguments: ts_morph_1.Writers.object({
                        providedIn: (writer) => writer.quote('root')
                    })
                },
                {
                    name: 'RxapOpenApiRemoteMethod',
                    arguments: parameter.options.inline ? (writer) => ts_morph_1.Writers.object({
                        serverId: parameter.options.serverId ? w => w.quote(parameter.options.serverId) : 'undefined',
                        operationId: w => w.quote(parameter.operationId),
                        operation: w => w.quote(JSON.stringify((0, schematics_open_api_1.GenerateParameterToOperationObjectWithMetadata)(parameter)).replace(/[\n\r\\]+/g, ''))
                    })(writer) : w => w.quote(operationId)
                }
            ],
            methods: [
                {
                    name: 'call',
                    parameters: callMethodParameters,
                    scope: ts_morph_1.Scope.Public,
                    returnType: `Promise<${responseType}>`,
                    hasOverrideKeyword: true,
                    statements: [
                        `return super.call(${withoutParametersAndRequestBody ? '' : 'parameters'});`
                    ]
                }
            ],
            extends: (writer) => {
                switch (parameter.options.transport) {
                    case 'amplify':
                        writer.write('AmplifyOpenApiRemoteMethod');
                        break;
                    default:
                        writer.write('OpenApiRemoteMethod');
                        break;
                }
                writer.write('<');
                writer.write(responseType);
                writer.write(', ');
                writer.write(parameterType);
                writer.write(', ');
                writer.write(requestBodyType);
                writer.write('>');
            },
            isExported: true
        };
        sourceFile.addClass(classStructure);
        const withoutParameters = (0, schematics_open_api_1.IsWithoutParameters)(parameter);
        if (!parameter.options.skipDirectives) {
            if ((0, schematics_open_api_1.IsCollectionResponse)(parameter)) {
                importStructures.push({
                    moduleSpecifier: '@rxap/utilities',
                    namedImports: [{ name: 'ArrayElement' }]
                });
                (0, create_directive_1.CreateDirective)({
                    filePath: fileName,
                    name: operationId,
                    prefix: parameter.options.prefix,
                    parametersType: `OpenApiRemoteMethodParameter<${parameterType}, ${requestBodyType}>`,
                    returnType: `ArrayElement<${responseType}>`,
                    template: true,
                    collection: true,
                    sourceFile,
                    withoutParameters
                });
            }
            (0, create_directive_1.CreateDirective)({
                filePath: fileName,
                name: operationId,
                prefix: parameter.options.prefix,
                parametersType: `OpenApiRemoteMethodParameter<${parameterType}, ${requestBodyType}>`,
                returnType: responseType,
                template: true,
                collection: false,
                sourceFile,
                withoutParameters
            });
            (0, create_directive_1.CreateDirective)({
                filePath: fileName,
                name: operationId,
                prefix: parameter.options.prefix,
                parametersType: `OpenApiRemoteMethodParameter<${parameterType}, ${requestBodyType}>`,
                returnType: responseType,
                template: false,
                collection: false,
                sourceFile,
                withoutParameters
            });
        }
        sourceFile.addImportDeclarations(importStructures);
        sourceFile.organizeImports({
            ensureNewLineAtEndOfFile: true
        });
    });
}
exports.GenerateRemoteMethod = GenerateRemoteMethod;
//# sourceMappingURL=generate-remote-method.js.map