"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const schematics_1 = require("@angular-devkit/schematics");
const path_1 = require("path");
const ts_morph_1 = require("ts-morph");
const schematics_ts_morph_1 = require("@rxap/schematics-ts-morph");
const schematics_open_api_1 = require("@rxap/schematics-open-api");
const schematics_utilities_1 = require("@rxap/schematics-utilities");
const generate_remote_method_1 = require("./generate-remote-method");
const const_1 = require("./const");
const coerce_open_api_project_1 = require("./coerce-open-api-project");
const generate_openapi_provider_1 = require("./generate-openapi-provider");
function GetOperationIdList(openapi) {
    const operationIdList = [];
    for (const [_, methods] of Object.entries(openapi.paths)) {
        if (methods) {
            for (const method of Object.keys(methods).filter(schematics_open_api_1.IsHttpMethod)) {
                const operation = methods[method];
                if ((0, schematics_open_api_1.IsOperationObject)(operation)) {
                    if ((0, schematics_open_api_1.IgnoreOperation)(['hidden'])(operation)) {
                        console.log(`Ignore operation '${operation.operationId}'`);
                    }
                    else {
                        if ((0, schematics_open_api_1.HasOperationId)(operation)) {
                            operationIdList.push(operation.operationId);
                        }
                    }
                }
            }
        }
    }
    return operationIdList;
}
function default_1(options) {
    return (host) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        var _a, _b;
        const openapi = yield (0, schematics_open_api_1.LoadOpenApiConfig)(host, options);
        const project = new ts_morph_1.Project({
            manipulationSettings: {
                indentationText: ts_morph_1.IndentationText.TwoSpaces,
                quoteKind: ts_morph_1.QuoteKind.Single,
                useTrailingCommas: true
            },
            useInMemoryFileSystem: true
        });
        if (!options.project) {
            options.project = 'open-api';
        }
        if (options.directory) {
            options.directory = options.directory.split('/').map(item => (0, schematics_utilities_1.dasherize)(item)).join('/');
        }
        const projectName = `${options.directory ? options.directory.split('/').join('-') + '-' : ''}${options.project}`;
        const projectBasePath = (0, schematics_utilities_1.HasProject)(host, projectName) ? (0, schematics_utilities_1.GetProjectSourceRoot)(host, projectName) : `libs/${options.directory ? options.directory + '/' : ''}${options.project}/src`;
        const basePath = (0, path_1.join)(projectBasePath, 'lib');
        options.prefix = (_b = (_a = options.prefix) !== null && _a !== void 0 ? _a : ((0, schematics_utilities_1.HasProject)(host, projectName) ? (0, schematics_utilities_1.GetProjectPrefix)(host, projectName) : (0, schematics_utilities_1.GetDefaultPrefix)(host))) !== null && _b !== void 0 ? _b : 'rxap';
        if (!options.debug) {
            // TODO : reset the hack after the schematic execution is finished
            console.debug = function () { };
        }
        const generatorFunctionList = [];
        const clearPathList = [];
        if (!options.skipRemoteMethod) {
            generatorFunctionList.push(generate_remote_method_1.GenerateRemoteMethod);
            clearPathList.push(const_1.REMOTE_METHOD_BASE_PATH);
        }
        return (0, schematics_1.chain)([
            (0, schematics_open_api_1.ClearOperation)([
                schematics_open_api_1.COMPONENTS_BASE_PATH,
                schematics_open_api_1.PARAMETER_BASE_PATH,
                schematics_open_api_1.RESPONSE_BASE_PATH,
                schematics_open_api_1.REQUEST_BODY_BASE_PATH,
                ...clearPathList,
            ], basePath),
            (0, coerce_open_api_project_1.CoerceOpenApiProject)(options.project, options.prefix, options.directory),
            () => tslib_1.__awaiter(this, void 0, void 0, function* () { yield (0, schematics_open_api_1.GenerateOperation)(openapi, project, options, generatorFunctionList); }),
            () => options.skipProvider ? (0, schematics_1.noop)() : (0, generate_openapi_provider_1.GenerateOpenapiProvider)(project, GetOperationIdList(openapi), options),
            (0, schematics_ts_morph_1.ApplyTsMorphProject)(project, basePath),
            (0, schematics_ts_morph_1.FixMissingImports)(),
            (tree) => {
                const indexFile = (0, path_1.join)(projectBasePath, 'index.ts');
                if (options.export) {
                    (0, schematics_utilities_1.CoerceFile)(tree, indexFile, (0, schematics_open_api_1.GenerateIndexExports)(project));
                }
                else {
                    (0, schematics_utilities_1.CoerceFile)(tree, indexFile, 'export {};');
                }
            },
        ]);
    });
}
exports.default = default_1;
//# sourceMappingURL=index.js.map